@@ -10,11 +10,11 @@ Crypt::LE - Let's Encrypt API interfacing module.
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -90,7 +90,7 @@ To import all of those, use C<':errors'> tag:
  $le->load_account_key('account.pem') == OK or die "Could not load the account key: " . $le->error_details;
  
 If you don't want to use error codes while checking whether the last called method has failed or not, you can use the
-rule of thumb that on success it will return zero. You can also call error() or error_description() methods, which
+rule of thumb that on success it will return zero. You can also call error() or error_details() methods, which
 will be set with some values on error.
 
 =cut    
@@ -626,7 +626,7 @@ For example, you can create a module like this to handle the challenge:
  
  sub handle_challenge_http {
   my $self = shift;
-  my $challenge = shift;
+  my ($challenge, $params) = @_;
   print "Domain: $challenge->{domain} expects '$challenge->{token}.$challenge->{fingerprint}' text in '$challenge->{token}' file\n";
   <STDIN>;
   return 1;
@@ -799,6 +799,23 @@ sub request_issuer_certificate {
     return $self->_status(ERROR, $content);
 }
 
+=head2 revoke_certificate($certificate_file)
+
+Revokes a certificate.
+
+B<Returns>: OK | READ_ERROR | ERROR.
+
+=cut
+
+sub revoke_certificate {
+    my $self = shift;
+	my $file = shift;
+	my $crt = $self->_file($file);
+    return $self->_status(READ_ERROR, "Could not read the certificate from '$file'.") unless $crt;
+	my ($status, $content) = $self->_request($self->{directory}->{'revoke-cert'}, { resource => 'revoke-cert', certificate => encode_base64url(Crypt::Format::pem2der($crt)) });
+    return ($status == SUCCESS) ? $self->_status(OK, "Certificate has been revoked.") : $self->_status(ERROR, $content);
+}
+
 #====================================================================================================
 # API Workflow helpers
 #====================================================================================================
