@@ -9,22 +9,41 @@ use Crypt::LE ':errors';
 sub info { print shift, "\n" }
 sub bail { &info; exit 255 }
 
-my $VERSION = '0.05';
+my $VERSION = '0.06';
 
 my ($account_key, $csr, $csr_key, $domains, $challenge_path, $crt, $live, $generate_missing, $generate_only, 
-    $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $verified, $debug, $help);
+    $handle_with, $handle_as, $handle_params, $complete_with, $complete_params, $revoke, $verified, $debug, $help);
 
 GetOptions ("key=s" => \$account_key, "csr=s" => \$csr, "csr-key=s" => \$csr_key, "domains=s" => \$domains, "path=s" => \$challenge_path,
             "crt=s" => \$crt, "generate-missing" => \$generate_missing, "generate-only" => \$generate_only, "handle-with=s" => \$handle_with,
 	    "handle-as=s" => \$handle_as, "handle-params=s" => \$handle_params, "complete-with=s" => \$complete_with, 
-            "complete-params=s" => \$complete_params, "live" => \$live, "debug" => \$debug, "verified" => \$verified, "help" => \$help);
+            "complete-params=s" => \$complete_params, "revoke" => \$revoke, "live" => \$live, "debug" => \$debug, "verified" => \$verified, 
+            "help" => \$help);
 
 my $usage = <<EOF;
 
-Usage example: le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing
+===============
+Usage examples: 
+===============
 
-Note: by default a staging server is used, which does not provide trusted certificates. This is to aviod hitting a rate limits on Let's Encrypt
-      live server. To generate an actual certificate, always add --live option.
+a) To register (if needed) and issue a certificate:
+
+   le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing
+
+b) To use external modules to handle challenges and process completion while getting a certificate:
+
+   le.pl --key account.key --csr domain.csr --csr-key domain.key --crt domain.crt --domains "www.domain.ext,domain.ext" --generate-missing \\
+         --handle-with Crypt::LE::Challenge::Simple --complete-with Crypt::LE::Complete::Simple
+
+   - See provided Crypt::LE::Challenge::Simple for an example of challenge handling module.
+   - See provided Crypt::LE::Complete::Simple for an example of completion handling module.
+
+c) To revoke a certificate:
+
+   le.pl --key account.key --crt domain.crt --revoke
+
+[!] NOTE: by default a staging server is used, which does not provide trusted certificates. This is to aviod hitting a 
+          rate limits on Let's Encrypt live server. To generate an actual certificate, always add --live option.
 
 Available parameters:
 
@@ -42,73 +61,79 @@ Available parameters:
  generate-only                    - Generate a new key and/or CSR if they are missing and then exit.
  crt <file>                       - Name for the domain certificate file.
  verified                         - Skip challenge verification steps (request/accept/verify). Use only if domains have been already verified recently.
+ revoke                           - Revoke a certificate.
  live                             - Connect to a live server instead of staging.
  debug                            - Print out debug messages.
  help                             - This screen.
 
-[ See Crypt::LE::Challenge::Simple for an example of challenge handling module. ]
-[ See Crypt::LE::Complete::Simple for an example of completion handling module. ]
-
 EOF
 
 info "\nCrypt::LE client v$VERSION started.\n";
 
 bail $usage if $help;
+my ($handler, $complete_handler);
 
-unless (($account_key and (-r $account_key or $generate_missing)) and
-        ($csr and (-r $csr or ($csr_key and $generate_missing)))) {
-    info "Incorrect parameters - need both an account key and CSR either loaded or generated.";
+unless ($account_key and (-r $account_key or $generate_missing)) {
+    info "Incorrect parameters - need an account key loaded or generated.";
     bail $usage;
 }
 
-if (!$generate_missing and ! -r $csr and (!$domains or $domains=~/^\s*$/)) {
-    bail "Domain list should be provided to generate a CSR.";
-}
-
 unless ($crt) {
-    bail "You would probably want to save the domain certificate somewhere - please specify a filename";
+    bail "Please specify a file name for the certificate.";
 }
 
-if ($challenge_path) {
-    bail "Path to save challenge files into should be a writable directory" unless (-d $challenge_path and -w _);
-}
-
-my ($handler, $complete_handler);
-$handle_as = $handle_as ? lc($handle_as) : 'http';
-
-if ($handle_with) {
-    eval "use $handle_with;";
-    bail "Cannot use the module to handle challenges with." if $@;
-    $handler = new $handle_with;
-    my $method = 'handle_challenge_' . $handle_as;
-    bail "Module to handle challenges does not seem to support the challenge type of '$handle_as'." unless $handler->can($method);
-    if ($handle_params) {
-	my $j = JSON->new->canonical()->allow_nonref();
-        eval {
-	    $handle_params = $j->decode($handle_params);
-        };
-        bail "Could not decode parameters for the challenge-handling module: $@" if $@;
-        bail "Please make sure that your 'handle-params' JSON document has {} around it." unless (ref $handle_params eq 'HASH');
-    }
+if ($revoke) {
+    bail "Need a certificate file for revoke to work." unless (-r $crt);
+    bail "Need an account key - revoke assumes you had a registered account when got the certificate." unless (-r $account_key);
 } else {
-    undef $handle_params;
-}
-
-if ($complete_with) {
-    eval "use $complete_with;";
-    bail "Cannot use the module to complete processing with." if $@;
-    $complete_handler = new $complete_with;
-    bail "Module to complete processing with does not seem to support the required 'complete' method." unless $complete_handler->can('complete');
-    if ($complete_params) {
-        my $j = JSON->new->canonical()->allow_nonref();
-        eval {
-            $complete_params = $j->decode($complete_params);
-        };
-        bail "Could not decode parameters for the completion module: $@" if $@;
-        bail "Please make sure that your 'complete-params' JSON document has {} around it." unless (ref $complete_params eq 'HASH');
+    unless ($csr and (-r $csr or ($csr_key and $generate_missing)))  {
+        bail "Incorrect parameters - need CSR loaded or generated.";
+    }
+
+    if (!$generate_missing and ! -r $csr and (!$domains or $domains=~/^\s*$/)) {
+        bail "Domain list should be provided to generate a CSR.";
+    }
+
+    if ($challenge_path) {
+        bail "Path to save challenge files into should be a writable directory" unless (-d $challenge_path and -w _);
+    }
+
+    $handle_as = $handle_as ? lc($handle_as) : 'http';
+
+    if ($handle_with) {
+        eval "use $handle_with;";
+        bail "Cannot use the module to handle challenges with." if $@;
+        $handler = new $handle_with;
+        my $method = 'handle_challenge_' . $handle_as;
+        bail "Module to handle challenges does not seem to support the challenge type of '$handle_as'." unless $handler->can($method);
+        if ($handle_params) {
+	    my $j = JSON->new->canonical()->allow_nonref();
+            eval {
+	        $handle_params = $j->decode($handle_params);
+            };
+            bail "Could not decode parameters for the challenge-handling module: $@" if $@;
+            bail "Please make sure that your 'handle-params' JSON document has {} around it." unless (ref $handle_params eq 'HASH');
+        }
+    } else {
+        undef $handle_params;
+    }
+
+    if ($complete_with) {
+        eval "use $complete_with;";
+        bail "Cannot use the module to complete processing with." if $@;
+        $complete_handler = new $complete_with;
+        bail "Module to complete processing with does not seem to support the required 'complete' method." unless $complete_handler->can('complete');
+        if ($complete_params) {
+            my $j = JSON->new->canonical()->allow_nonref();
+            eval {
+                $complete_params = $j->decode($complete_params);
+            };
+            bail "Could not decode parameters for the completion module: $@" if $@;
+            bail "Please make sure that your 'complete-params' JSON document has {} around it." unless (ref $complete_params eq 'HASH');
+        }
+    } else {
+        undef $complete_params;
     }
-} else {
-    undef $complete_params;
 }
 
 # Begin work
@@ -125,6 +150,12 @@ if (-r $account_key) {
     bail "Failed to save an account key file" if _write($account_key, $le->account_key);
 }
 
+if ($revoke) {
+    bail $le->error_details if $le->revoke_certificate($crt);
+    info "Certificate has been revoked.";
+    exit 0;
+}
+
 if (-r $csr) {
     info "Loading a CSR from $csr";
     $le->load_csr($csr, $domains) == OK or bail "Could not load a CSR: " . $le->error_details;
